﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;

namespace GitCommands
{


    [XmlRoot("dictionary")]

    public class XmlSerializableDictionary<TKey, TValue>
            : Dictionary<TKey, TValue>, IXmlSerializable
    {

        #region IXmlSerializable Members

        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
            XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));
            bool wasEmpty = reader.IsEmptyElement;
            
            
            
            reader.Read();
            if (reader.NodeType == System.Xml.XmlNodeType.XmlDeclaration)
                reader.Read();
            if (wasEmpty || reader.IsEmptyElement)
                return;
            if (reader.ReadToDescendant("item"))
            {
                while (reader.NodeType != System.Xml.XmlNodeType.EndElement)
                {

                    reader.ReadStartElement("item");
                    reader.ReadStartElement("key");

                    TKey key = (TKey)keySerializer.Deserialize(reader);
                    reader.ReadEndElement();


                    reader.ReadStartElement("value");
                    TValue value = (TValue)valueSerializer.Deserialize(reader);
                    reader.ReadEndElement();




                    this[key] = value;

                    reader.ReadEndElement();

                    reader.MoveToContent();

                }
            }
            reader.ReadEndElement();

        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {

            XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
            XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));



            foreach (TKey key in from k in this.Keys orderby k select k)
            {
                writer.WriteStartElement("item");


               
                writer.WriteStartElement("key");
                keySerializer.Serialize(writer, key);
                writer.WriteEndElement();



                writer.WriteStartElement("value");
                TValue value = this[key];
                valueSerializer.Serialize(writer, value);
                writer.WriteEndElement();



                writer.WriteEndElement();

            }


        }
        #endregion

    }
}